<?php
require_once dirname(__FILE__) . '/../vendor/autoload.php';
require_once dirname(__FILE__) . '/vCard-parser/vCard.php';
require_once dirname(__FILE__) . '/CardDAV-PHP/carddav.php';

use Brick\PhoneNumber\PhoneNumberType;
use \libphonenumber\NumberParseException;

//DO NOT MOVE THE THIS OR THE CONFIG.INI FILE, THE POSITION IS HARCODED IN THREE METHODS BELOW
class phoneMiddleware
{
	const GITHUB_MAIN_URL = "https://github.com/Massi-X/freepbx-phonemiddleware/";
	const GITHUB_RELEASE_URL = self::GITHUB_MAIN_URL."releases/latest/";
	const GITHUB_UPDATE_URL = "https://raw.githubusercontent.com/Massi-X/freepbx-phonemiddleware/main/module.xml";
	const TYPE_PLAIN = 'plain';
	const TYPE_XML = 'xml';

	//for backgroundHelper + cache + vcard and xml functions
	const MODE_PHONEBOOK_REFRESH = 0;
	const MODE_RELOAD_VCARD = 1;
	//for backgroundHelper only
	const MODE_CHECK_UPDATES = 99;

	//if new are added remember to update setter and getXMLforPhones obviously
	const PHONE_TYPE_NO_LIMITS = 0;
	const PHONE_TYPE_FANVIL = 1;
	const PHONE_TYPE_SNOM = 2;

	private $carddav = null; //backend
	private $tmpdirname = '/phonemiddleware/';
	private $vcardfilename = 'vcards.xml';
	private $phonebookfilename = 'phonebook.xml';

	//config but only used by scripts, not stored
	private $header_type = self::TYPE_XML;

	//config
	private $INI_FILE;
	private $url = null;
	private $username = null;
	private $password = null;
	private $cache_expire = 10;
	private $country_code = 'IT';
	private $output_construct = array("fn");
	private $phone_type = self::PHONE_TYPE_NO_LIMITS;
	private $update_check = true;
	private $last_update = -1;

	/**
	 * Constructor
	 * Init with constructor deprecated. Use init()
	 */
	public function __construct()
	{
		//create tmp folder
		if (!is_dir(sys_get_temp_dir() . $this->tmpdirname))
			mkdir(sys_get_temp_dir() . $this->tmpdirname);

		$this->INI_FILE = realpath(__DIR__ . '/..') . '/config.ini.php'; //beacuse this will always run inside DOCUMENT_ROOT, there is no need to include it there (the class is used in CLI too and here this variable does not exist)
		$this->set_config(parse_ini_file($this->INI_FILE));

		//update check. This does not check for errors.
		if ($this->update_check && time() - $this->last_update > 604800 && date("N") == 1) { //7 * 24 * 60 * 60 (1 week). Monday only
			//store before to stop duplicate calls
			$this->last_update = time();
			$this->store_config($this->get_config());
			//start check
			exec("php -f " . __DIR__ . "/backgroundHelper.php " . self::MODE_CHECK_UPDATES . " > /dev/null &");  //beacuse this will always run inside DOCUMENT_ROOT, there is no need to include it there (the class is used in CLI too and here this variable does not exist)
		}
	}

	/**
	 * Ininitialize the carddav backend
	 */
	public function init()
	{
		$this->carddav = new carddav_backend($this->url);
		$this->carddav->set_auth($this->username, $this->password);
	}

	/**
	 * Get an array with the currently chosen configuration
	 * 
	 * @return array	the associative array with configuration data
	 */
	public function get_config()
	{
		return array(
			'carddav_url' => $this->url,
			'carddav_user' => $this->username,
			'carddav_psw' => $this->password,
			'cache_expire' => $this->cache_expire,
			'country_code' => $this->country_code,
			'output_construct' => implode("/", $this->output_construct),
			'phone_type' => $this->phone_type,
			'update_check' => $this->update_check,
			'last_update' => $this->last_update
		);
	}

	/**
	 * Set an array with the currently chosen configuration (it's not needed to set all the parameters).
	 * Alternative to setting all the parameters one by one
	 * WARNING! THIS DOES NOT SAVE THE CONFIG, FOR THAT USE store_config() instead
	 * 
	 * @return array	the associative array with configuration data
	 */
	private function set_config($config)
	{
		$this->set_url($config['carddav_url']);
		$this->set_auth($config['carddav_user'], $config['carddav_psw']);
		$this->set_cache_expire($config['cache_expire']);
		$this->set_country_code($config['country_code']);
		$this->set_output_construct($config['output_construct']);
		$this->set_phone_type($config['phone_type']);

		//private-only settable values
		if (isset($config['update_check'])) $this->update_check = !!$config['update_check'];
		$this->last_update = $config['last_update'];

		return $this->get_config();
	}

	/**
	 * Store configuration in the ini file
	 * 
	 * @param   array	$assoc_arr	The associative array to be stored
	 * @return  boolean	true on success, false otherwise
	 */
	public function store_config($assoc_arr)
	{
		$content = ";<?php die(); ?>\n";

		foreach ($assoc_arr as $key => $elem) {
			if (is_array($elem)) {
				for ($i = 0; $i < count($elem); $i++)
					$content .= $key . "[] = \"" . $elem[$i] . "\"\n";
			} else if ($elem == "") $content .= $key . " = \n";
			else $content .= $key . " = \"" . $elem . "\"\n";
		}

		if (!$handle = fopen($this->INI_FILE, 'w')) return false;

		$success = fwrite($handle, $content);
		fclose($handle);

		if($success) $this->set_config(parse_ini_file($this->INI_FILE)); //update class preferences to make sure everythig remains in sync
		return $success;
	}

	/**
	 * Get module version
	 * 
	 * @return string	version number
	 */
	public function get_version()
	{
		return simplexml_load_file(realpath(__DIR__ . '/../..') . "/admin/modules/phonemiddleware/module.xml")->version; //beacuse this will always run inside DOCUMENT_ROOT, there is no need to include it there (the class is used in CLI too and here this variable does not exist)
	}

	/**
	 * Check if an update is available
	 * 
	 * @return int	the new online version number, or false if no update is available
	 */
	public function check_module_update()
	{
		try {
			$onlineVersion = simplexml_load_file(self::GITHUB_UPDATE_URL)->version;
			if (version_compare($this->get_version(), $onlineVersion) === -1) return $onlineVersion;
		} catch (Exception $e) {
			//don't care
		}

		return false;
	}

	/**
	 * Sets type for "die" errors
	 *
	 * @param	string	$type	One of TYPE_PLAIN or TYPE_XML. Default TYPE_PLAIN
	 * @return	void
	 */
	public function set_header_type($type)
	{
		if ($type != self::TYPE_PLAIN && $type != self::TYPE_XML) return;

		$this->header_type = $type;
	}

	/**
	 * Sets time to cache the requests
	 *
	 * @param	int	$time	Time in minutes to cache the requests (0 to disable). Default: 10
	 * @return	void
	 */
	public function set_cache_expire($time)
	{
		if (!is_numeric($time)) return;

		if ($time < 0)
			$time = 0;

		$this->cache_expire = $time;
	}

	/**
	 * Sets time to cache the requests
	 *
	 * @param	string	$code	Country code (default = IT). If the number is given with + country code at the start, it's ignored
	 * @return	void
	 */
	public function set_country_code($code)
	{
		if (empty($code)) return;

		$this->country_code = $code;
	}

	/**
	 * Sets the output for numberToCNAM
	 *
	 * @param   string	$construct	Formatted string of parameters divided by '/' (supports everything in https://tools.ietf.org/id/draft-ietf-vcarddav-vcardrev-02.html + supports sub parameter selection by adding (...) at the end of the parameter). Defaults to 'fn'
	 * @return	void
	 */
	public function set_output_construct($construct)
	{
		if (empty($construct)) return;

		$this->output_construct = explode('/', $construct);
	}

	/**
	 * Sets phone type (Fanvil, Snom, No limitations...)
	 *
	 * @param	string	$type	Phone type (default = No limitation)
	 * @return	void
	 */
	public function set_phone_type($type)
	{
		if ($type != self::PHONE_TYPE_NO_LIMITS && $type != self::PHONE_TYPE_FANVIL && $type != self::PHONE_TYPE_SNOM) return;

		$this->phone_type = $type;
	}

	/**
	 * Sets carddav url
	 *
	 * @param	string	$url	CardDAV server url
	 * @return	void
	 */
	public function set_url($url)
	{
		$this->url = $url;
	}

	/**
	 * Sets authentication information
	 *
	 * @param	string	$username	CardDAV server username
	 * @param	string	$password	CardDAV server password
	 * @return	void
	 */
	public function set_auth($username, $password)
	{
		$this->username = $username;
		$this->password = $password;
	}

	/**
	 * Request to delete the cache after updating it
	 *
	 * @param	int		$mode	The mode used for refreshing the cache
	 * @return	void
	 */
	public function request_clear_cache($mode = -1)
	{
		if ($mode != self::MODE_PHONEBOOK_REFRESH && $mode != self::MODE_RELOAD_VCARD) return false;

		$fileName = sys_get_temp_dir() . $this->tmpdirname . 'update-mode' . $mode . '.lock';

		if (file_exists($fileName)) //update already in progress
			return false;

		return file_put_contents($fileName, $mode) ? true : false;
	}

	/**
	 * Gets all vCards in XML format for Phones and print them
	 *
	 * @param	bool	$forceUpdate	Wheter to force the update of the database
	 */
	public function getXMLforPhones($forceUpdate = false)
	{
		$file = $this->cache_or_get_file(sys_get_temp_dir() . $this->tmpdirname, $this->phonebookfilename, null, $this->cache_expire);
		$execUpdate = false;

		//check if the file is intact
		if (!$forceUpdate && !$this->isXMLContentValid($file['content'])) $forceUpdate = true;

		if (!$forceUpdate && $file['valid']) return $file['content']; //file is valid (not expired)
		else if (!$forceUpdate && $file['content'] != null && $file['content'] != '') $execUpdate = true; //file is expired so we exec a background update (but return the old one anyway)

		if ($execUpdate) {
			exec("php -f helpers/backgroundHelper.php " . self::MODE_PHONEBOOK_REFRESH . " " . $this->url . ' ' . $this->username . " " . $this->password . " " . $this->cache_expire . " " . $this->phone_type . " > /dev/null &");
			return $file['content'];
		}

		//the file doesn't exist or forceupdate = true
		//this is an expensive process, but it should always be executed in the background as long as there is some cache available
		$file = sys_get_temp_dir() . $this->tmpdirname . 'tempXMLWriter.xml';
		$simplified_xml = new XMLWriter();
		fopen($file, "w");
		$simplified_xml->openURI($file);
		$simplified_xml->setIndent(4);
		$simplified_xml->startDocument('1.0', 'utf-8');
		$simplified_xml->startElement('IPPhoneDirectory');

		$bookArray = array();
		$lockFile = sys_get_temp_dir() . $this->tmpdirname . 'update-mode' . self::MODE_PHONEBOOK_REFRESH . '.lock';

		try {
			$data = simplexml_load_string($this->getAllVcards(true)); //force update of vcards too
		} catch (Exception $e) {
			$errStr1 = "Something went wrong while retrieving datas from the carddav server. Please make sure your credentials and url are correct. Below the HTTP error code.";
			if ($this->header_type == self::TYPE_PLAIN) echo $errStr1 . "<br>" . $e->getMessage();
			else {
				$xml = new SimpleXMLElement('<xml/>');
				$xml->addChild('error', $errStr1);
				$xml->addChild('error', $e->getMessage());
				print($xml->asXML());
			}

			//delete the lock
			if (file_exists($lockFile)) unlink($lockFile);
			die();
		}

		foreach ($data->element as $item) {
			$vCard = new vCard(false, $item->vcard); //vcard cannot be null, so no checks

			$phoneNumbers = $this->arrayTelWalk($vCard->tel);
			if (empty($phoneNumbers)) continue;

			$res = $this->getNameByConstruct($vCard); //retrieve the name using the provided construct

			$tmpArray = array(
				'name' => $res,
				'phone' => $phoneNumbers
			);

			array_push($bookArray, $tmpArray);
		}

		$this->aasort($bookArray, "name");
		$phoneNumberUtil = \libphonenumber\PhoneNumberUtil::getInstance();

		foreach ($bookArray as $card) {
			$simplified_xml->startElement('DirectoryEntry');
			$simplified_xml->writeElement('Name', $card['name']);

			/* Used only for phones with limitations */
			$iterations = 0;
			$otherName = "Other";

			if ($this->phone_type == self::PHONE_TYPE_SNOM) $otherName = "Office";

			$usedFields = array(
				"Telephone" => false,
				"Mobile" => false,
				$otherName => false
			);

			/* Used only for phones with limitations */
			if ($this->phone_type == self::PHONE_TYPE_FANVIL || $this->phone_type == self::PHONE_TYPE_SNOM) {
				if (count($card['phone']) == 1) {
					//Sorry, you have a very stupid phone... You can't have a card with only the mobile number so we change it into a standard telephone
					$simplified_xml->writeElement('Telephone', $card['phone'][0]);
					$usedFields["Telephone"] = true;
				} else {
					//this can give some problems if there are duplicates, but for now it works
					$lastElement = end($card['phone']);
					foreach ($card['phone'] as $number) {
						//stop our directory after using all the tags
						if ($iterations == 3) continue;

						//make sure our stupid phone always get a <telephone> element
						if ($number == $lastElement && !$usedFields["Telephone"]) {
							$simplified_xml->writeElement('Telephone', $number);
							$usedFields["Telephone"] = true;
							continue;
						}

						try {
							$type = $phoneNumberUtil->getNumberType($phoneNumberUtil->parse($number, $this->country_code));
							$writeTo = $otherName;
							switch ($type) {
								case PhoneNumberType::FIXED_LINE:
									$writeTo = "Telephone";
									break;
								case PhoneNumberType::MOBILE:
									$writeTo = "Mobile";
									break;
							}
						} catch (NumberParseException $e) {
							//this is not a valid number, simply reject it
							continue;
						}

						//check for stupid phones to make sure they receive exactly three different elements. Logic is sorted by best match possible
						if ($writeTo == "Telephone") {
							if (!$usedFields["Telephone"]) {
								$simplified_xml->writeElement('Telephone', $number);
								$usedFields["Telephone"] = true;
							} else if (!$usedFields[$otherName]) {
								$simplified_xml->writeElement($otherName, $number);
								$usedFields[$otherName] = true;
							} else if (!$usedFields["Mobile"]) {
								$simplified_xml->writeElement('Mobile', $number);
								$usedFields["Mobile"] = true;
							}
						} else if ($writeTo == "Mobile") {
							if (!$usedFields["Mobile"]) {
								$simplified_xml->writeElement('Mobile', $number);
								$usedFields["Mobile"] = true;
							} else if (!$usedFields["Telephone"]) {
								$simplified_xml->writeElement('Telephone', $number);
								$usedFields["Telephone"] = true;
							} else if (!$usedFields[$otherName]) {
								$simplified_xml->writeElement($otherName, $number);
								$usedFields[$otherName] = true;
							}
						} else if ($writeTo == $otherName) {
							if (!$usedFields[$otherName]) {
								$simplified_xml->writeElement($otherName, $number);
								$usedFields[$otherName] = true;
							} else if (!$usedFields["Telephone"]) {
								$simplified_xml->writeElement('Telephone', $number);
								$usedFields["Telephone"] = true;
							} else if (!$usedFields["Mobile"]) {
								$simplified_xml->writeElement('Mobile', $number);
								$usedFields["Mobile"] = true;
							}
						}
						$iterations++;
					}
				}
			} else if ($this->phone_type == self::PHONE_TYPE_NO_LIMITS) {
				foreach ($card['phone'] as $number) {
					try {
						$type = $phoneNumberUtil->getNumberType($phoneNumberUtil->parse($number, $this->country_code));
						switch ($type) {
							case PhoneNumberType::FIXED_LINE:
								$simplified_xml->writeElement('Telephone', $number);
								break;
							case PhoneNumberType::MOBILE:
								$simplified_xml->writeElement('Mobile', $number);
								break;
							default:
								$simplified_xml->writeElement('Other', $number); //is this the correct tag? No way to test it myself
								break;
						}
					} catch (NumberParseException $e) {
						//this is not a valid number, simply reject it
					}
				}
			}
			if ($this->phone_type == self::PHONE_TYPE_SNOM && $iterations < 3) { //Snom' phones require all tags to be always present
				foreach ($usedFields as $field => $value) {
					if (!$value) $simplified_xml->writeElement($field, '');
				}
			}
			$simplified_xml->endElement();
		}
		$simplified_xml->endElement();
		$simplified_xml->endDocument();
		$simplified_xml->flush();

		//get the data and delete temp file
		$data = file_get_contents($file);
		unlink($file);

		//delete the lock
		if (file_exists($lockFile)) unlink($lockFile);

		//create the new file
		$this->cache_or_get_file(sys_get_temp_dir() . $this->tmpdirname, $this->phonebookfilename, $data, $this->cache_expire);
		return $data;
	}

	/**
	 * Take a number and return caller ID (if found)
	 *
	 * @param	int	$callerNumber	The number
	 */
	public function getCNFromPhone($callerNumber)
	{
		$vCard = null;

		if ($callerNumber == null || $callerNumber == "") //if for some reason the number is empty, return empty
			return $callerNumber;

		$callerNumber = preg_replace('/[^\d]/i', "", $callerNumber); //replace everything that's not a digit and "/"

		try {
			$data = simplexml_load_string($this->getAllVcards());
		} catch (Exception $e) {
			return "[W!] " . $callerNumber; //cannot retrieve the vcards! Be sure to check your settings!
		}

		foreach ($data->element as $item) {
			preg_match_all('/^.*TEL\D+([\d\s]+)$/im', $item->vcard, $matches); //match TEL elements
			$matches = "/" . implode($matches[1], "/") . "/"; //put them all in a string separated by "/"
			$matches = preg_replace('/[^\/\d]/i', "", $matches); //replace everything that's not a digit and "/"

			//if there is a match, save the vcard we need
			if (preg_match('/\/' . $callerNumber . '\//i', $matches)) {
				$vCard = new vCard(false, $item->vcard);
				break;
			}
		}

		$res = $this->getNameByConstruct($vCard); //retrieve the name using the provided construct
		return $res != null ? $res : $callerNumber; //if everything fails, simply return the number untouched
	}

	/**
	 * Create a formatted string from the vcard paramters using the construct
	 *
	 * @param	Object	$vCard	The vCard object
	 * @return	string			Formatted name or null if vcard is null. In case of malformed construct, probably nothing will be printed
	 */
	private function getNameByConstruct($vCard)
	{
		if ($vCard == null)
			return null;
		
		$result = "";

		$isNfilled = in_array("n", $this->output_construct); //if 'n' is in the construct
		$isFNfilled = in_array("fn", $this->output_construct); //if 'fn' is in the construct. No need to check empty for this, is always filled

		if ($isNfilled) //if 'n' is in the construct and the value is not empty
			$isNfilled = !empty(array_filter($vCard->n[0]));

		foreach ($this->output_construct as $parameter) {
			preg_match_all('/\((.*)\)/U', $parameter, $sub); //find sub-parameters inside brackets and put them in $sub

			if (empty($sub[1])) {
				$value = $vCard->{$parameter}[0];

				if ($parameter == "tel") //special case only for numbers (multidimensional array)
					$value = $this->arrayTelWalk($vCard->{$parameter});

				if (is_array($value)) { //array to string + delete empty values

					foreach ($value as $name => $key) { //delete multidimensional array from values (not supported)
						if (is_array($key)) unset($value[$name]);
					}

					if ($parameter == "n") $value = implode(" ", array_filter($value)); //for 'n' (names) we should not add a semicolon
					else $value = implode(", ", array_filter($value)); //for everything else a colon is sufficient
				}

				if (empty($value)) continue; //do not insert empty values

				if ($value[strlen($value) - 1] == " ") $value = substr($value, 0, -1); //strip end space if present

				if ($parameter == "fn" || $parameter == "n" || (!$isNfilled && !$isFNfilled && $parameter == "org")) //'fn' and 'n' are not inside brackets. If both are empty and 'org' is present in the construct, this will also be printed without brackets
					$result .= $value;
				else $result .= "(" . $value . ")";

				$result .= " ";
			} else {
				$parameter = substr($parameter, 0, stripos($parameter, '(')); //delete sub from parameter name

				foreach ($sub[1] as $key) {
					$value = $vCard->{$parameter}[0][$key];
					if (!empty($value)) {
						if (!$isNfilled && !$isFNfilled && $parameter == "org") $result .= $value . " ";
						else $result .= "(" . $value . ") ";
					}
				}
			}
		}
		return substr($result, 0, -1); //strip the last space
	}

	/**
	 * Build the file needed for creating the xml or getting the number
	 *
	 * @param	bool	$forceUpdate	If this is a call from backgroundHelper.php
	 * @return	string					File is valid or not (bool) and the content
	 */
	public function getAllVcards($forceUpdate = false)
	{
		$vcards = $this->cache_or_get_file(sys_get_temp_dir() . $this->tmpdirname, $this->vcardfilename, null, $this->cache_expire);
		$execUpdate = false;

		//check if the file is intact
		if (!$forceUpdate && !$this->isXMLContentValid($vcards['content'])) $forceUpdate = true;

		if (!$forceUpdate && $vcards['valid']) return $vcards['content']; //file is valid (not expired)
		else if (!$forceUpdate) $execUpdate = true; //file is expired so we exec a background update (but return the old one anyway)

		if ($execUpdate) {
			exec("php -f helpers/backgroundHelper.php " . self::MODE_RELOAD_VCARD . " " . $this->url . " " . $this->username . " " . $this->password . " " . $this->cache_expire . " > /dev/null &");
			return $vcards['content'];
		}

		$lockFile = sys_get_temp_dir() . $this->tmpdirname . 'update-mode' . self::MODE_RELOAD_VCARD . '.lock';

		//the file doesn't exist or forceupdate = true. This can throw an exception, be sure to handle it.
		try {
			$data = $this->carddav->get();

			//delete the lock
			if (file_exists($lockFile)) ($lockFile);

			$vcards = $this->cache_or_get_file(sys_get_temp_dir() . $this->tmpdirname, $this->vcardfilename, $data, $this->cache_expire);
			return $vcards['content'];
		} catch (Exception $e) {
			//delete the lock
			if (file_exists($lockFile)) unlink($lockFile);

			throw $e; //pass exception to caller
		}
	}

	/**
	 * Query the the cache for a file, if exists and it is valid return it else return it with a warning
	 * if a content is given, store it anyway
	 *
	 * @param	string	$file		Folder name, the path will bel relative to this file
	 * @param	string	$file		File name
	 * @param	string	$content	The content to update or null to get it
	 * @param	int		$minutes	Time duration to cache the file
	 * @return	array				File is valid or not (bool) and the content
	 */
	private function cache_or_get_file($folder, $fileName, $content = null, $minutes = 60)
	{
		if (!is_dir($folder)) mkdir($folder);

		$current_time = time();
		$expire_time = $minutes * 60;

		if ($content != null) {
			$content .= '<!-- cached:  ' . time() . '-->';
			file_put_contents($folder . '/' . $fileName, $content);
			return array(
				'valid' => false,
				'content' => file_get_contents($folder . '/' . $fileName)
			);
		}

		if (file_exists($folder . '/' . $fileName)) {
			$file_time = filemtime($folder . '/' . $fileName);
			if ($current_time - $file_time < $expire_time) {
				return array(
					'valid' => true,
					'content' => file_get_contents($folder . '/' . $fileName)
				);
			} else {
				return array(
					'valid' => false,
					'content' => file_get_contents($folder . '/' . $fileName)
				);
			}
		} else {
			fopen($folder . '/' . $fileName, "w");
			touch($folder . '/' . $fileName, $current_time - $expire_time);
			return array(
				'valid' => false,
				'content' => null
			);
		}
	}

	/**
	 * Sort array in ascending order
	 *
	 * @param	array	$array	The array
	 * @param	string	$key	The key to use for sorter
	 * @return	string			The sorted array
	 */
	private function aasort(&$array, $key)
	{
		$sorter = array();
		$ret = array();
		reset($array);
		foreach ($array as $ii => $va) $sorter[$ii] = $va[$key];
		asort($sorter);
		foreach ($sorter as $ii => $va) $ret[$ii] = $array[$ii];
		$array = $ret;
	}
	/**
	 * Walk through arrays and get telephone datas
	 *
	 * @param	array	$arr	A multidimensional array
	 * @return	array			A simple array with all the phones
	 */
	private function arrayTelWalk($arr)
	{
		if ($arr == null) return null;

		$result = array();
		$arr = array_values($arr);
		$length = count($arr);

		for ($i = 0; $i < $length; $i++) {
			$object = $arr[$i];
			if (is_array($object)) {
				$innerRes = $this->arrayTelWalk($object);
				foreach ($innerRes as $value) array_push($result, $value);
			} else {
				//First strip everything but numbers, + and *
				$object = preg_replace('/[^0-9\+\*]/', '', $object);
				//Then replace + with 00 for better compatibility with asterisk
				$object = preg_replace('/^\+/', '00', $object);
				if ($object != '') array_push($result, $object);
			}
		}
		return $result;
	}

	/**
	 * @author 	Francesco Casula <fra.casula@gmail.com>
	 * @param 	string $xmlContent A well-formed XML string
	 * @param 	string $version 1.0
	 * @param 	string $encoding utf-8
	 * @return 	bool
	 */
	private function isXMLContentValid($xmlContent, $version = '1.0', $encoding = 'utf-8')
	{
		if (trim($xmlContent) == '') return false;

		libxml_use_internal_errors(true);

		$doc = new DOMDocument($version, $encoding);
		$doc->loadXML($xmlContent);

		$errors = libxml_get_errors();
		libxml_clear_errors();

		return empty($errors);
	}
}
?>